;(function( $ ){

    var methods = {
    
        // create new instance of orgBuilder
        init : function( options ) { 
            
            // default options may be overidden
            opts = $.extend( {
                'stackLeaves': true,
                'draggable': false,
                'tooltips': true,
                'showMinimized': true,
                'minAfterLevel': null,
                'nodeHeight': 55,
                'nodeWidth': 94,
                'expandHeight': 175,
                'expandWidth': 300
            }, options);

            // execute code for each element in jQuery selector used for plugin call
            // returning a function maintains jQuery chainability
            return this.each(function() {
			
                // keep current element in global $this variable
                $this = $( this );
                
                var $nodes = $this.find( '.node' );
                
                var $root = $this.find( '#root' ).children( '.node' );

                methods.collectLeaves();
                methods.initDialog();
                
                if ( opts.tooltips ) {
                    methods.initTooltip( $this );
                    methods.bindHover( $nodes );
                }
                
                if ( opts.draggable ) {
                    methods.bindDrag( $nodes );
                    methods.bindDrop( $nodes );
                }
                
                if ( opts.stackLeaves ) {
                    methods.collectStack();
                }
                
                $this.find( '.label' ).height( opts.nodeHeight - 10 );
                $this.find( '.detail' ).height( opts.expandHeight - $this.find( '.label' ).outerHeight() - 42 );
                methods.showNode( $root );
                
                if ( opts.minAfterLevel === null ){
                    methods.showDescendents( $root );
                } else {
                    $this.find( '.node' ).each(function(){
                        if ( methods.findNodeGeneration( $(this) ) > opts.minAfterLevel ){
                            $(this).addClass( 'min' ).bind( 'click.min', function(e){
                                if ( $( e.target ).attr( 'class' ) !== 'connect' ){
                                    $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
                                    methods.showNode( $(this), null, function(){ methods.connectNodes(); } );
                                }
                            });
                        }
                    });
                    methods.hideNode( $this.find( '.min' ) );
                    methods.showNode( $this.find( '.node' ).not( '.min' ), null, function(){ methods.connectNodes(); } );
                }
                
                // initialize jQuery UI buttons                
                $( '.dialog-button,.menu-button' ).button();
                $this.disableSelection();
                
            });
        },
        // attach jQuery UI draggable behavior to node(s) 
        bindDrag : function( node ) {
            node.not( $( '#root' ).children( '.node' ) ).draggable({
				helper: 'clone',
				delay: 50,
				zIndex: 50,
                revert: true,
                revertDuration: 250,
                cancel: '.connect',
				start: function( event, ui ) {
                    $this.find( '.menu' ).hide();
                    var progeny = methods.findDescendentNodes( $( this ) ).add( $( this ) );
                    progeny.droppable( 'disable' ).css( 'opacity', 0.4 ).find( '.connect' ).css( 'opacity', 0.4 );
                    ui.helper.css( 'opacity', 1.0 ).find('.connect').hide();
				},
				stop: function( event, ui ) {
                    $this.find( '.node' ).css( 'opacity', 1.0 ).find( '.connect' ).css( 'opacity', 1.0 );
                    $this.find( '.node' ).not( '.min' ).droppable( 'enable' );
                    $this.find( '.node' ).not( '.expanded, .min' ).draggable( 'enable' );
				}
			});
        },
        // attach jQuery droppable behavior to node(s)
        bindDrop : function( node ) {
            node.droppable({
				hoverClass: 'drop-hover',
				accept: '.node',
				tolerance: 'intersect',
				drop: function( event, ui ) {					
                    $( '.connect' ).remove();
                    var whenDone = function(){
                        family.appendTo( $( '.drop' ).parent( '.family' ).children( '.offspring' ) );
                        methods.collectStack();
                        methods.collectLeaves();
                        methods.showNode( nodes, null, function(){ 
                            if ( family.find( '.expanded' ).length > 0 ){
                                methods.expandNode( family.find( '.expanded' ) );
                            } else {
                                methods.connectNodes();
                            }
                        } );
                        $('.drop').removeClass( 'drop' );
                    };
                    var family = ui.draggable.parent( '.family' );
                    var nodes = family.find( '.node' ).not( '.min' );
                    $( this ).addClass( 'drop' );
                    methods.hideNode( nodes, whenDone );
					ui.helper.remove();
				}
			});
        },
        // show option menu when icon is clicked
        bindClick : function( node ) {
            node.find( '.opt' ).bind( 'click.orgBuilder', function(e) {
                $this.find( '.tooltip' ).hide();
                var menuX = e.pageX - $this.parent().offset().left + 5
                var menuY = e.pageY - $this.parent().offset().top + 5
                if ( !$.browser.webkit || !$this.parent().is( 'body' ) ){
                    menuX += $this.parent().scrollLeft();
                    menuY += $this.parent().scrollTop();
                }
                $this.find( '.menu' ).fadeIn( 200 ).css({
                    top: menuY,
                    left: menuX
                });           
                // store node clicked globally
                nodeClicked = $( this ).parent( '.header' ).parent( '.content' ).parent( '.node' );
                return false;
            });
        },
        // attach tooltip behavior
        bindHover : function( node ) {
            var optHover = false;
            var nodeHover = false;
            // node hover
            node.find( '.content' ).hover(function(){
                nodeHover = true;
                if ( $this.not( ':animated' ) ){
                    if ( optHover ) {
                        $this.find('.tooltip').text( 'Options' ).show();
                    } else {
                        $this.find('.tooltip').text( $( this ).find( '.label' ).text() ).show();
                    }
                }
            },
            function(){
                nodeHover = false;
                $this.find('.tooltip').hide();
            });
            // option icon hover
            node.find( '.opt' ).hover(function(){
                optHover = true;
                if ( $this.not( ':animated' ) ){
                    $this.find('.tooltip').text( 'Options' ).show();
                }
            },
            function(){
                optHover = false;
                if ( nodeHover ) {
                    $this.find('.tooltip').text( $( this ).parent().parent().find( '.label' ).text() );
                } else {
                    $this.find('.tooltip').hide();
                }
            });
            // detail area hover
            node.find( '.detail' ).hover(function(){
                $this.find('.tooltip').hide();
            },
            function(){
                $this.find('.tooltip').show();
            });
        },
        // create new node and append to specified parent node
        createNode : function( parent, name ) { 
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            $('<div class="family"><div class="new-node node" name="' + name + '"><div class="content"><div class="header"><img class="opt" src="orgBuilder/images/Gear.svg.hi.png" /></div><div class="label"></div></div></div><div class="offspring" /></div>')
                .appendTo( parent.parent( '.family' ).children( '.offspring' ) )
                .show()
                .find( '.label' ).text( name );
            if ( opts.tooltips ) {
                methods.bindHover( $('.new-node') );
            }
            if ( opts.draggable ) {
                methods.bindDrag( $('.new-node') );
                methods.bindDrop( $('.new-node') );
            }
            if ( $this.find( '.menu' ).find( '.menu-button' ).length ) {
                $( '.new-node' ).find( '.opt' ).show();
            }
            methods.bindClick( $('.new-node') );
            methods.collectStack();
            methods.collectLeaves();
            var whenDone = function(){
                methods.connectNodes();
            };
			methods.showNode( $('.new-node'), { 'width':opts.nodeWidth, 'height':opts.nodeHeight }, whenDone );
            $('.new-node').removeClass( 'new-node' );
        },
        // replace node name attribute and label text
        renameNode : function( node, name ) {
            node.attr( 'name', name ).find('.label').text( name );
        },
        // remove node (and any descendents) from orgchart
        deleteNode : function( node ) { 
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            var whenDone = function(){
                node.parent( '.family' ).remove();
                methods.collectStack();
                methods.collectLeaves();
                methods.connectNodes();
            };
            var progeny = methods.findDescendentNodes( node ).add( node );
            methods.hideNode( progeny, whenDone );
        },
        // enlarge node to expand dimensions in opts
        expandNode : function( node ) {
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            var content = node.find( '.content' );
            node.addClass( 'expanded' ).draggable( 'disable' ).add( content ).animate({ 
                width:opts.expandWidth,
                height:opts.expandHeight
            }, {
                duration: 1000,
                step: function(now, fx) {
                    $this.width( $( '#root' ).width() + 30 );
                },
                complete: function() {
                    methods.connectNodes();
                }
            });
        },
        // collapse node to default dimensions in opts
        collapseNode : function( node ) {
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            var content = node.find( '.content' );
            node.removeClass( 'expanded' ).draggable( 'enable' ).add( content ).animate({ 
                width:opts.nodeWidth,
                height:opts.nodeHeight
            }, {
                duration: 1000,
                step: function(now, fx) {
                    $this.width( $( '#root' ).width() + 15 );
                },
                complete: function() {
                    methods.connectNodes();
                }
            });
        },
        // show hidden node(s)
        showNode : function( node, dim, callback ) {
            var nodeCount = node.length;
            var attr = $.extend( {
                'width':opts.nodeWidth,
                'height':opts.nodeHeight
            }, dim);
            var content = node.find( '.content' );
            var parentWidth = 'root';
            node.removeClass( 'min' ).unbind( 'click.min' ).draggable( 'enable' ).droppable( 'enable' ).animate({ 
                width:attr.width,
                height:attr.height
            }, {
                duration: 1000,
                step: function(now, fx) {
                    $this.width( $( '#root' ).width() + 45 );
                },
                complete: function(){
                    content.css({ 'width':opts.nodeWidth, 'height':opts.nodeHeight, 'opacity':1.0 });
                    nodeCount--;
                    if ( callback && nodeCount === 0 ){
                        callback.call( undefined );
                    }
                }
            })
            if ( $this.find( '.menu' ).find( '.menu-button' ).length ) {
                node.find( '.opt' ).show();
            }
        },
        // hide node(s)
        hideNode : function( node, callback ) {
            var nodeCount = node.length;
            var content = node.find( '.content' );
            var nodeWidth = 10;
            var nodeHeight = 10;
            content.css( 'opacity', 0 );
            node.draggable( 'disable' ).add( content ).animate({ 
                width:nodeWidth,
                height:nodeHeight
            }, {
                duration: 1000,
                step: function(now, fx) {
                    $this.width( $this.find( '#root' ).width() + 15 );
                },
                complete: function(){
                    nodeCount--;
                    if ( callback && nodeCount === 0 ){
                        callback.call( undefined );
                    }
                }
            }).find( '.opt' ).hide();
        },
        // show children nodes of specified node
        showChildren : function( node, dim, callback, preventRedraw ) {
            var children = methods.findChildNodes( node );
            var attr = $.extend( {
                'width': opts.nodeWidth,
                'height': opts.nodeHeight
            }, dim);
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            methods.showNode( children, attr );
            children.promise().done( function() {
                if ( !preventRedraw ) {
                    methods.connectNodes();
                }
                if ( callback ) {
                    callback.call( undefined );
                }
            });
        },
        // show all descendent nodes of specified node
        showDescendents : function( node, dim, callback, preventRedraw ) {
            var children = methods.findChildNodes( node );
            var attr = $.extend( {
                'width': opts.nodeWidth,
                'height': opts.nodeHeight
            }, dim);
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            children.each(function(){
                searchChildren( $( this ) );
            });
            children.promise().done( function() { 
                if ( !preventRedraw ) {
                    methods.connectNodes(); 
                }
                if ( callback ) {
                    callback.call( undefined );
                }
            });
            function searchChildren( node ){
                methods.showNode( node, attr );
                var children = methods.findChildNodes( node );
                children.each(function(){
                    searchChildren( $( this ) );
               });
            }
        },
        // hide all descendent nodes of specified node
        hideDescendents : function( node, callback, preventRedraw ) {
            var progeny = methods.findDescendentNodes( node );
            $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
            node.addClass( 'atRest' );
            progeny.removeClass( 'expanded' ).addClass( 'min' ).unbind( 'click.min' ).bind( 'click.min', function(e){
                if ( $( e.target ).attr( 'class' ) !== 'connect' ){
                    $( '.connect' ).stop().animate( { 'opacity': 0 }, 500 );
                    methods.showNode( $(this), null, function(){ methods.connectNodes(); } );
                }
            });
            methods.hideNode( progeny );
            progeny.promise().done(function(){
                if ( !preventRedraw ) {
                    methods.connectNodes();
                }
                if ( callback ) {
                    callback.call( undefined );
                }
            });
        },
        // create new option in node option menu and assign function to call on click event
        addOption : function( name, func ) { 
            // if context menu container doesn't exist, create it
            if ( $this.find( '.menu' ).length === 0 ) {
                methods.initContext( $this );
                methods.bindClick( $this.find( '.node' ) );
                $this.find( '.node:not(.min)' ).find( '.opt' ).show();
                var newHght = $this.find( '.label' ).height() - 18;
                $this.find( '.label' ).height( newHght );
            }
            // append menu button to custom context menu
            $('<div />')
                .addClass( 'menu-button' )
                .button()
                .text( name )
                .bind( 'click.orgBuilder', func )
                .appendTo( $this.find( '.menu' ) );
        },
        // draw connectors between nodes
        connectNodes : function() {
            $this.find( '.connect' ).remove();
            var ua = $.browser;
            $this.find( '.node' ).not( '.node:first' ).each(function(){       
                var node = $( this ),
                nodeBorder = parseInt( node.css( 'border-left-width' ).replace( /em|px/ig, '' ) ),
                nodeMargin = parseInt( node.css( 'margin-left' ).replace( /em|px/ig, '' ) ),
                nodePadding = parseInt( node.css( 'padding-top' ).replace( /em|px/ig, '' ) ),
                nodeHeight = node.outerHeight(),
                nodeWidth = node.outerWidth(),
                nodeLeftX = node.offset().left,
                nodeMidX = nodeLeftX + nodeWidth/2,
                parent = methods.findParentNode( node ),
                parentHeight = parent.outerHeight(),
                parentWidth = parent.outerWidth(),
                parentLeftX = parent.offset().left,
                parentMidX = parentLeftX + parentWidth/2,
                parentTopY = parent.offset().top,
                parentBottomY = parentTopY + parentHeight,
                prevNode = node.parent( '.family' ).prev( '.family' ).children( '.node' ),
                prevHeight = prevNode.outerHeight(),
                prevWidth = prevNode.outerWidth(),
                uaFix = 0,
                radiusFix = 5;
                if ( prevNode.length ){
                    prevTopY = prevNode.offset().top;
                }
                if ( ua.msie ) {
                    parentLeftX -= 1;
                    parentMidX -= 1;
                    parentTopY -= 1;
                    parentBottomY -= 1;
                    nodeLeftX -= 1;
                    nodeMidX -= 1;
                    uaFix = 2;
                }
                /* conditions for horizontal node arrangement:
                    (1) stackLeaves option is off
                    (2) this node or one of its siblings has children 
                    (3) this node has no visible siblings
                */
                if ( ! opts.stackLeaves || ! methods.isLeafGeneration( node ) || methods.findSiblingNodes( node ).length === 1 ){
                    // create connectors for horizontally aligned node
                    if ( parentMidX > nodeMidX ){
                        if ( methods.findSiblingNodes( node ).length === 1 ){
                            $('<div />')
                            .addClass( 'connect-left connect' )
                            .appendTo( node )
                            .offset({ left:nodeMidX, top:parentBottomY + nodeMargin })
                            .width( parentMidX - nodeMidX - radiusFix )
                            .height( nodeMargin )
                            .css( 'opacity', 0 );
                        } else {
                             $('<div />')
                             .addClass( 'connect-left connect' )
                             .appendTo( node )
                             .offset({ left:nodeMidX, top:parentBottomY + nodeMargin })
                             .width( parentMidX - nodeMidX )
                             .height( nodeMargin )
                             .css( 'opacity', 0 );
                        }
                    } else {
                        $('<div />')
                        .addClass( 'connect-right connect' )
                        .appendTo( node )
                        .offset({ left:parentMidX, top:parentBottomY + nodeMargin })
                        .width( nodeMidX - parentMidX )
                        .height( nodeMargin )
                        .css( 'opacity', 0 );
                    }
                    if ( node.parent().is( node.parent( '.family' ).parent( '.offspring' ).children( '.family' ).last() ) ){
                        var prevGen = methods.findParentNode( node ).parent( '.family' ).children( '.offspring' ).children( '.family' );
                        var lastNode = prevGen.last();
                        var lastMidX = lastNode.offset().left + lastNode.outerWidth()/2
                        if ( parentMidX > lastMidX + 5 ){
                            $('<div />')
                            .addClass( 'connect-top connect' )
                            .appendTo( node )
                            .offset({ left:parentMidX - 5, top:parentBottomY })
                            .width( 5 )
                            .height( nodeMargin )
                            .css({ 'opacity': 0, '-moz-border-radius-bottomright': '5px', 'border-bottom-right-radius': '5px' });
                        } else {
                            $('<div />')
                            .addClass( 'connect-top connect' )
                            .appendTo( node )
                            .offset({ left:parentMidX, top:parentBottomY - uaFix })
                            .height( nodeMargin + uaFix )
                            .css( 'opacity', 0 );
                        }
                    }
                } else {
                    // create connectors for vertically aligned node
                    var maxWidth = 5;
                    methods.findSiblingNodes( node ).each(function(){
                       if ( $(this).outerWidth() > maxWidth ){
                            maxWidth = $(this).outerWidth();
                        }                       
                    });
                    var diffWidth = ( maxWidth - nodeWidth ) / 2;
                    if ( node.parent().is( node.parent( '.family' ).parent( '.offspring' ).children( '.family' ).first() ) ){
                        $('<div />')
                        .addClass( 'connect-top-leaf connect' )
                        .appendTo( node )
                        .offset({ left:nodeLeftX - diffWidth - nodeMargin, top:parentBottomY + nodeMargin })
                        .width( diffWidth + nodeMargin - 1 )
                        .height( nodeHeight/2 + nodeMargin - uaFix )
                        .css( 'opacity', 0 );
                        $('<div />')
                        .addClass( 'connect-parent-leaf connect' )
                        .appendTo( node )
                        .offset({ left:nodeLeftX - diffWidth - nodeMargin, top:parentBottomY + nodeMargin })
                        .width( diffWidth + nodeWidth/2 + nodeMargin - radiusFix - 1 )
                        .css( 'opacity', 0 );
                        $('<div />')
                        .addClass( 'connect-top connect' )
                        .appendTo( node )
                        .offset({ left:parentMidX - 5, top:parentBottomY - uaFix })
                        .height( nodeMargin + uaFix )
                        .width( 5 )
                        .css({ 'opacity': 0, '-moz-border-radius-bottomright': '5px', 'border-bottom-right-radius': '5px' });
                    } else {
                        $('<div />')
                        .addClass( 'connect-leaf connect' )
                        .appendTo( node )
                        .offset({ left:nodeLeftX - diffWidth - nodeMargin, top:prevTopY + prevHeight/2 - radiusFix })
                        .width( diffWidth + nodeMargin - 1 )
                        .height( nodeHeight/2 + prevHeight/2 + nodeMargin*2 + radiusFix - uaFix )
                        .css( 'opacity', 0 );
                    }
                }
            });
            $( '.connect' ).animate( { 'opacity': 1.0 }, 1500 );
        },
        // add class 'stacked' to all leaf nodes
        collectStack : function() {
            $this.find( '.stacked' ).removeClass( 'stacked' );
            if ( opts.stackLeaves ) {
                $this.find( '.offspring' ).each(function(){
                    if ( $( this ).children( '.family' ).children( '.offspring' ).children( '.family' ).children( '.node' ).length === 0 && methods.findNodeGeneration( $( this ).children( '.family' ).children( '.node:first' ) ) !== 2 ){
                        $( this ).children( '.family' ).children( '.node' ).parent( '.family' ).addClass( 'stacked' );
                    }
                });
            }
        },
        // add class 'leaf' to all leaf nodes (applies gray background)
        collectLeaves : function() {
            $this.find( '.node' ).not( '.node:first' ).each(function(){ 
                if ( methods.isLeafNode( $( this ) ) ){
                    $( this ).addClass( 'leaf' );
                } else {
                    $( this ).removeClass( 'leaf' );
                }
            });
        },
        // create tooltip and attach to cursor
        initTooltip : function() {           
            $('<div />').addClass( 'tooltip' ).prependTo( $this );
            $this.bind( 'mousemove.orgBuilder', function(e) {
                var topPos = -27, 
                    leftPos = 8,
                    winExtX = $this.parent().width() + $(document).scrollLeft() + $this.parent().offset().left - $this.find( '.tooltip' ).width() - 45;
                if ( e.pageX >  winExtX ) {
                    leftPos = -20 - $this.find( '.tooltip' ).width();
                }
                var tooltipX = e.pageX + leftPos - $this.parent().offset().left;
                var tooltipY = e.pageY + topPos - $this.parent().offset().top;
                if ( !$.browser.webkit || !$this.parent().is( 'body' ) ){
                    tooltipX += $this.parent().scrollLeft();
                    tooltipY += $this.parent().scrollTop();
                } 
                $this.find( '.tooltip' ).css({
                    'top': tooltipY,
                    'left': tooltipX
                });                    
            });
        },
        // create option menu container
        initContext : function() {
            $('<div />').addClass( 'menu' ).appendTo( $this );              
            // hide option menu on click event
            $(document).bind( 'click.orgBuilder', function() {
                $this.find( '.menu' ).hide();
            });
        },
        // initialize jQuery UI dialog
        initDialog : function() {
            if ( $('.dialog-orgBuilder').length === 0 ){
                var dialogHTML = '<div class="msg" /><input type="text" class="field" size="30"></input><div class="dialog-button" />';
                $('<div />').addClass( 'dialog-orgBuilder' ).append( dialogHTML ).appendTo( 'body' ).dialog({
                    modal: true,
                    autoOpen: false,
                    height: 155
                });
            }
        },
        // open dialog with specified options
        openDialog : function( params ) {
            var params = $.extend( {
                'input' : false,
                'inputValue' : null,
                'button' : false,
                'buttonAction' : methods.closeDialog
            }, params);
            $( '.dialog-button' ).unbind( 'click.orgBuilder' );
            $( '.ui-dialog-title' ).text( params.title );
            $( '.dialog-orgBuilder' )
                .dialog( 'open' )
                .find( '.msg' )
                .html( params.bodyText );
            if ( params.button ) {
                $( '.dialog-button' )
                    .text( params.buttonText )
                    .show()
                    .bind( 'click.orgBuilder', params.buttonAction );
            } else {
                $( '.dialog-button' ).hide();
            }
            if ( params.input ) {
                $( '.field' ).show().val( params.inputValue );
            } else {
                $( '.field' ).hide();
            }
        },
        // close dialog
        closeDialog : function() {
            $( '.dialog-orgBuilder' ).dialog ( 'close' );
        },
        // destroy plugin
        destroy : function() {
            $( window ).unbind('.orgBuilder');
        },
        // returns a jQuery object representing the immediate parent node of the given node
		findParentNode : function( node ) {
			return node.parent( '.family' ).parent( '.offspring' ).parent( '.family' ).children( '.node' );				
		},
        // returns a jQuery object representing the immediate child nodes of the given node
		findChildNodes : function( node ) {
			return node.parent( '.family' ).children( '.offspring' ).children( '.family' ).children( '.node' );
		},
        // returns a jQuery object representing all descendent nodes of the given node
		findDescendentNodes : function( node ) {
			return node.parent( '.family' ).children( '.offspring' ).find( '.node' );
		},
        // return a jQuery object representing all sibling nodes of the given node
        findSiblingNodes : function( node ) {
            return methods.findChildNodes( methods.findParentNode( node ) );
        },
	    // returns an array of jQuery objects representing all visible leaf nodes beneath the given node
		findLeafNodes : function( node ) {
			var leaves = [];
			methods.findDescendentNodes(node).each(function() {
				if ( $( this ).parent( '.family' ).children( '.offspring' ).children( '.family' ).children( '.node' ).length === 0 ) {
					leaves.push( $( this ) );
				}
			});
			return leaves;
		},
        // returns an integer representing the given node's level in the hierarchy, starting with 1 for the root node and increasing by one per 'generation'
        findNodeGeneration : function( node ) {
            var level = 1;
            function countNextLevel( node ) {
                if ( node.parent( '.family' ).parent( 'offspring' ).parent( '.family' ).children( '.node' ).length > 0 ) {
                    node = methods.findParentNode( node );
                    level ++;
                    countNextLevel( node );
                }
            }
            countNextLevel( node );
            return level;
        },
        // returns boolean whether given node is hidden.  leaf nodes are considered always expanded
        ishiddenNode : function( node ) {
            if ( methods.findDescendentNodes( node ).length > 0 || methods.isLeafNode( node ) ) {
                return false;
            } else {
                return true;
            }
        },
        // returns boolean whether given node is a leaf
        isLeafNode : function( node ) {
            if ( methods.findChildNodes( node ).length > 0 ) {
                return false;
            } else {
                return true;
            }
        },
        // returns boolean whether given node's siblings are all leaves
        isLeafGeneration : function( node ) {
            if ( node.parent( '.family' ).parent( '.offspring' ).children( '.family' ).children( '.offspring' ).children( '.family' ).children( '.node' ).length > 0 ){
                return false;
            } else {
                return true;
            }
        }
    };

    // jQuery plugin creation
    $.fn.orgBuilder = function( method ) {
    
        // Method calling logic
        if ( methods[method] ) {
            return methods[ method ].apply( this, Array.prototype.slice.call( arguments, 1 ));
        } else if ( typeof method === 'object' || ! method ) {
            return methods.init.apply( this, arguments );
        } else {
            $.error( 'Method ' +  method + ' does not exist in orgBuilder.' );
        }    
  
    };

})( jQuery );